#include <stdio.h>
#include <glib.h>

#include "gm-call-handler.h"


gboolean system_handler (GmCallHandler *call_handler,
			 GmCallInfo *call_info,
			 gpointer handler_id) {
  gboolean myreturn = FALSE;
  g_debug ("SYSTEM-HANDLER: triggered, returning %s", myreturn?"TRUE":"FALSE");

  return myreturn;
}


gboolean foo_handler (GmCallHandler *call_handler,
		      GmCallInfo *call_info,
		      gpointer handler_id) {
  gboolean myreturn = TRUE;
  g_debug ("FOO-HANDLER: triggered, returning %s", myreturn?"TRUE":"FALSE");
  call_info->results = GM_CALL_RESULT_REJECTED | GM_CALL_RESULT_TERMINATING;
  g_debug ("FOO-HANDLER: unregistering myself");
  gm_call_handler_unregister_handler (call_handler, handler_id);
  return myreturn;
}


gboolean bar_handler (GmCallHandler *call_handler,
                      GmCallInfo *call_info,
		      gpointer handler_id) {
  gboolean myreturn = FALSE;
  g_debug ("BAR-HANDLER: triggered, returning %s", myreturn?"TRUE":"FALSE");

  return myreturn;
}


gboolean dialog_handler (GmCallHandler *call_handler,
			 GmCallInfo *call_info,
			 gpointer handler_id) {
  gboolean myreturn = FALSE;
  g_debug ("DIALOG-HANDLER: triggered, returning %s", myreturn?"TRUE":"FALSE");

  return myreturn;
}


int main (int argc, char ** argv) {
  GmCallHandler *call_handler = NULL;
  GmEndpointIdentifier *local_ep = NULL,
		       *remote_ep = NULL,
		       *xfer_ep = NULL;
  GmCallInfo *call_info = NULL;

  /* make GLib typing system happier */
  g_type_init ();

  call_handler = gm_call_handler_new ();

  if (!call_handler) {
    fprintf (stderr, "Could not create call handler, exiting!\n");
    return 1;
  }

  call_info = gm_call_handler_new_call_info ();

  local_ep = g_new (GmEndpointIdentifier, 1);
  remote_ep = g_new (GmEndpointIdentifier, 1);
  xfer_ep = g_new (GmEndpointIdentifier, 1);

  gm_call_handler_set_uniq_token (call_handler, call_info);
  call_info->protocol = GM_CALL_PROTOCOL_SIP;
  call_info->direction = GM_CALL_IN;
  call_info->from = remote_ep;
  call_info->to = local_ep;
  call_info->transferrer = NULL;

  gm_call_handler_register_handler (call_handler, GM_CALL_IN,
				    GM_CALL_HANDLER_PRIO_LOWEST,
				    &system_handler, NULL);

  gm_call_handler_register_handler (call_handler, GM_CALL_IN,
				    GM_CALL_HANDLER_PRIO_LOWEST,
				    &dialog_handler, NULL);

  gm_call_handler_register_handler (call_handler, GM_CALL_IN,
				    GM_CALL_HANDLER_PRIO_NORMAL,
				    &foo_handler, NULL);

  gm_call_handler_register_handler (call_handler, GM_CALL_IN,
				    GM_CALL_HANDLER_PRIO_DEFAULT,
				    &bar_handler, NULL);

  g_debug ("----------- STEP 1: IN call");
  gm_call_handler_handle (call_handler, call_info);
  g_debug ("----------- STEP 2: IN call with unregistered FOO handler");
  gm_call_handler_handle (call_handler, call_info);
}
